exports.readContentTypesFromXml = readContentTypesFromXml;
exports.defaultContentTypes = {
    findContentType: function(path) {
        return null;
    }
};


function readContentTypesFromXml(xml) {
    var extensionDefaults = {};
    var overrides = {};
    
    xml.root.children.forEach(function(child) {
        if (child.name === "content-types:Default") {
            extensionDefaults[child.attributes.Extension] = child.attributes.ContentType;
        }
        if (child.name === "content-types:Override") {
            var name = child.attributes.PartName;
            if (name.charAt(0) === "/") {
                name = name.substring(1);
            }
            overrides[name] = child.attributes.ContentType;
        }
    });
    
    return {
        findContentType: function(path) {
            var overrideContentType = overrides[path];
            if (overrideContentType) {
                return overrideContentType;
            } else {
                var pathParts = path.split(".");
                var extension = pathParts[pathParts.length - 1];
                return extensionDefaults[extension];
            }
        }
    };
}
